/*
 * (C) Copyright 2003
 * Author : Hamid Ikdoumi (Atmel)
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <at91rm9200_net.h>
#include <configs/at91rm9200dk.h>  /*RAJOUTE PAR MOI */
#include <net.h>
#include <dm9161.h>

#ifdef CONFIG_LXT971_NO_SLEEP
#include <miiphy.h>
#endif

/*RAJOUTER PAR MOI*/
static void lxt971_no_sleep(AT91PS_EMAC p_mac)
{
	unsigned short reg;
	printf("LXT971NOSLEEP\n");
	at91rm9200_EmacEnableMDIO (p_mac);
	at91rm9200_EmacReadPhy (p_mac, PHY_LXT971_PORT_CFG, &reg);
	printf("VALEUR DE REG AVANT MASK: %x\n",reg);
	reg &= ~0x0040;                  /* disable sleep mode */
	printf("VALEUR DE REG APRES MASK: %x\n",reg);
	at91rm9200_EmacWritePhy (p_mac, PHY_LXT971_PORT_CFG, &reg);
	at91rm9200_EmacDisableMDIO (p_mac);
}
/* CONFIG_LXT971_NO_SLEEP */

/*RAJOUTE PAR MOI*/

#ifdef CONFIG_DRIVER_ETHER

#if (CONFIG_COMMANDS & CFG_CMD_NET)

/*
 * Name:
 *	dm9161_IsPhyConnected
 * Description:
 *	Reads the 2 PHY ID registers
 * Arguments:
 *	p_mac - pointer to AT91S_EMAC struct
 * Return value:
 *	TRUE - if id read successfully
 *	FALSE- if error
 */
static unsigned int dm9161_IsPhyConnected (AT91PS_EMAC p_mac)
{
	unsigned short Id1, Id2;
/*DEBUG*/
printf("Fonction IsPhyConnected\n\n");
	at91rm9200_EmacEnableMDIO (p_mac);
	at91rm9200_EmacReadPhy (p_mac, PHY_COMMON_ID1, &Id1);
	at91rm9200_EmacReadPhy (p_mac, PHY_COMMON_ID2, &Id2);
	at91rm9200_EmacDisableMDIO (p_mac);

printf("Id1: %x\n",Id1);
printf("Id2: %x\n",Id2);

	if (((Id1 << 16 ) | (Id2 & 0xfff0)) == PHY_LXT971A_ID)
	/* On sur ID1 le PHY ID NUMBER > 0013
	Sur ID2, on recherche PHY ID NUMBER 78
	Manufacturer Model Number E
	Revision Number qui ne nous importe pas 
	car on utilise un masque pour coller  toutes les sries */	
	{
	printf("PHY detecte comme CONNECTED!!! ^^ O_o ^^\n\n");
	return TRUE;
	}else
	{
	printf("Unknown PHY\n");
	return FALSE;
	}
}

/*
 * Name:
 *	dm9161_GetLinkSpeed
 * Description:
 *	Link parallel detection status of MAC is checked and set in the
 *	MAC configuration registers
 * Arguments:
 *	p_mac - pointer to MAC
 * Return value:
 *	TRUE - if link status set succesfully
 *	FALSE - if link status not set
 */
static UCHAR dm9161_GetLinkSpeed (AT91PS_EMAC p_mac)
{
	unsigned short stat1, stat2;

printf("Fonction GetLinkSpeed\n");

	if (!at91rm9200_EmacReadPhy (p_mac, PHY_COMMON_STAT, &stat1)){
printf("\tRetourne FALSE\n\n");
		return FALSE;}

printf("*****\nVALEUR STAT1: %x\n*****\n",stat1);
printf("Fonction GetLinkSpeed\n");
printf("Boucle Test Link Status Up?\n");
	if (!(stat1 & PHY_COMMON_STAT_LNK_STAT))	/* link status up? */
	{
printf("\tRetourne FALSE\n\n");
		return FALSE;}
	else printf("\tRetourne TRUE/RIEN\n\n");

printf("Fonction GetLinkSpeed\n");
printf("Boucle Test EmacReadPhy STAT2\n");
	if (!at91rm9200_EmacReadPhy (p_mac, PHY_LXT971_STAT2, &stat2))
	{		
printf("\tRetourne FALSE\n\n");
		return FALSE;}
	else printf("\tRetourne TRUE/RIEN\n\n");
printf("*****\nVALEUR STAT2: %x\n*****\n",stat2);

	if ((stat1 & PHY_COMMON_STAT_100BXFD) \
	&& (stat2 & PHY_LXT971_STAT2_DUPLEX_MODE) \
	&& (stat2 & PHY_LXT971_STAT2_100BTX)) {
		/*set Emac for 100BaseTX and Full Duplex  */
		p_mac->EMAC_CFG |= AT91C_EMAC_SPD | AT91C_EMAC_FD;
printf("Connexion a 100BaseT FullDuplex\n"); /*OK*/
		return TRUE;}

	if ((stat1 & PHY_COMMON_STAT_10BTFD) \
	&& ((stat2 & 0x0200) == PHY_LXT971_STAT2_DUPLEX_MODE) \
	&& ((stat2 & 0x4000) == 0x0000/*PHY_LXT971_STAT2_100BTX*/)) {
		/*set MII for 10BaseT and Full Duplex  */
		p_mac->EMAC_CFG = (p_mac->EMAC_CFG &
				~(AT91C_EMAC_SPD | AT91C_EMAC_FD))
				| AT91C_EMAC_FD;
printf("Connexion a 10BaseT FullDuplex\n");
		return TRUE;}

	if ((stat1 & PHY_COMMON_STAT_100BXHD) \
	&& ((stat2 & 0x0200) == 0x0000 /*PHY_LXT971_STAT2_DUPLEX_MODE*/) \
	&& ((stat2 & 0x4000) == PHY_LXT971_STAT2_100BTX)) {
		/*set MII for 100BaseTX and Half Duplex  */
		p_mac->EMAC_CFG = (p_mac->EMAC_CFG &
				~(AT91C_EMAC_SPD | AT91C_EMAC_FD))
				| AT91C_EMAC_SPD;
printf("Connexion a 100BaseT HalfDuplex\n");
		return TRUE;}

	if ((stat1 & PHY_COMMON_STAT_10BTHD) \
	&& ((stat2 & 0x0200) == 0x0000 /*PHY_LXT971_STAT2_DUPLEX_MODE*/) \
	&& ((stat2 & 0x0200) == 0x0000 /*PHY_LXT971_STAT2_100BTX*/)) {
		/*set MII for 10BaseT and Half Duplex  */
		p_mac->EMAC_CFG &= ~(AT91C_EMAC_SPD | AT91C_EMAC_FD);
printf("Connexion a 10BaseT HalfDuplex\n");
		return TRUE;}
printf("PHY_LXT971_STAT2_DUPLEX_MODE: %x\n", PHY_LXT971_STAT2_DUPLEX_MODE);
printf("PHY_LXT971_STAT2_100BTX: %x\n",PHY_LXT971_STAT2_100BTX);
	return FALSE;
}


/*
 * Name:
 *	dm9161_InitPhy
 * Description:
 *	MAC starts checking its link by using parallel detection and
 *	Autonegotiation and the same is set in the MAC configuration registers
 * Arguments:
 *	p_mac - pointer to struct AT91S_EMAC
 * Return value:
 *	TRUE - if link status set succesfully
 *	FALSE - if link status not set
 */
static UCHAR dm9161_InitPhy (AT91PS_EMAC p_mac)
{
	UCHAR ret = TRUE;
	unsigned short IntValue;
/*DEBUG*/
printf("Fonction InitPhy\n");

	/*************RAJOUTE PAR MOI*/
	lxt971_no_sleep(p_mac);
	/*************RAJOUTE PAR MOI*/	
	
	at91rm9200_EmacEnableMDIO (p_mac);
	
		if (!dm9161_GetLinkSpeed (p_mac)) {
		/* Try another time */
		ret = dm9161_GetLinkSpeed (p_mac);
	}
	
	/* Disable PHY Interrupts */
	at91rm9200_EmacReadPhy (p_mac, PHY_LXT971_INT_ENABLE, &IntValue);
	/* clear FDX, SPD, Link, INTR masks */
	IntValue &= ~(PHY_LXT971_INT_ENABLE_DUPLEXMSK | PHY_LXT971_INT_ENABLE_SPEEDMSK |
		      PHY_LXT971_INT_ENABLE_LINKMSK | (PHY_LXT971_INT_ENABLE_INTEN & 0));
	at91rm9200_EmacWritePhy (p_mac, PHY_LXT971_INT_STATUS, &IntValue);
	at91rm9200_EmacDisableMDIO (p_mac);
	return (ret);
}


/*
 * Name:
 *	dm9161_AutoNegotiate
 * Description:
 *	MAC Autonegotiates with the partner status of same is set in the
 *	MAC configuration registers
 * Arguments:
 *	dev - pointer to struct net_device
 * Return value:
 *	TRUE - if link status set successfully
 *	FALSE - if link status not set
 */
static UCHAR dm9161_AutoNegotiate (AT91PS_EMAC p_mac, int *status)
{
	unsigned short value;
	unsigned short PhyAnar;
	unsigned short PhyAnalpar;
/*DEBUG*/
printf("Fonction AutoNegociate\n\n\n");
	/* Set dm9161 control register */
	if (!at91rm9200_EmacReadPhy (p_mac, PHY_COMMON_CTRL, &value))
		return FALSE;
	value &= ~PHY_COMMON_CTRL_AUTO_NEG;	/* remove autonegotiation enable */
	value |= PHY_COMMON_CTRL_ISOLATE;	/* Electrically isolate PHY */
	if (!at91rm9200_EmacWritePhy (p_mac, PHY_COMMON_CTRL, &value))
		return FALSE;

	/* Set the Auto_negotiation Advertisement Register */
	/* MII advertising for Next page, 100BaseTxFD and HD, 10BaseTFD and HD, IEEE 802.3 */
	PhyAnar = PHY_COMMON_AUTO_ADV_NP | PHY_COMMON_AUTO_ADV_100BTXFD | PHY_COMMON_AUTO_ADV_100BTX |
		  PHY_COMMON_AUTO_ADV_10BTFD | PHY_COMMON_AUTO_ADV_10BT | PHY_COMMON_AUTO_ADV_802_3;
	if (!at91rm9200_EmacWritePhy (p_mac, PHY_COMMON_AUTO_ADV, &PhyAnar))
		return FALSE;

	/* Read the Control Register     */
	if (!at91rm9200_EmacReadPhy (p_mac, PHY_COMMON_CTRL, &value))
		return FALSE;

	value |= PHY_COMMON_CTRL_SPD_MA | PHY_COMMON_CTRL_AUTO_NEG | PHY_COMMON_CTRL_DUPLEX;
	if (!at91rm9200_EmacWritePhy (p_mac, PHY_COMMON_CTRL, &value))
		return FALSE;
	/* Restart Auto_negotiation  */
	value |= PHY_COMMON_CTRL_RES_AUTO;
	if (!at91rm9200_EmacWritePhy (p_mac, PHY_COMMON_CTRL, &value))
		return FALSE;

	/*check AutoNegotiate complete */
	udelay (10000);
	at91rm9200_EmacReadPhy (p_mac, PHY_COMMON_STAT, &value);
	if (!(value & PHY_COMMON_STAT_AN_COMP))
		return FALSE;

	/* Get the AutoNeg Link partner base page */
	if (!at91rm9200_EmacReadPhy (p_mac, PHY_COMMON_AUTO_LNKB, &PhyAnalpar))
		return FALSE;

	if ((PhyAnar & PHY_COMMON_AUTO_LNKB_100BTXFD) && (PhyAnalpar & PHY_COMMON_AUTO_LNKB_100BTXFD)) {
		/*set MII for 100BaseTX and Full Duplex  */
		p_mac->EMAC_CFG |= AT91C_EMAC_SPD | AT91C_EMAC_FD;
		return TRUE;
	}

	if ((PhyAnar & PHY_COMMON_AUTO_LNKB_10BTFD) && (PhyAnalpar & PHY_COMMON_AUTO_LNKB_10BTFD)) {
		/*set MII for 10BaseT and Full Duplex  */
		p_mac->EMAC_CFG = (p_mac->EMAC_CFG &
				~(AT91C_EMAC_SPD | AT91C_EMAC_FD))
				| AT91C_EMAC_FD;
		return TRUE;
	}
	return FALSE;
}


/*
 * Name:
 *	at91rm92000_GetPhyInterface
 * Description:
 *	Initialise the interface functions to the PHY
 * Arguments:
 *	None
 * Return value:
 *	None
 */
void at91rm92000_GetPhyInterface(AT91PS_PhyOps p_phyops)
{
/*DEBUG*/
printf("Fonction GetPhyInterface\n\n");
	p_phyops->Init = dm9161_InitPhy;
/*printf("dm9161_InitPhy: %x\n",dm9161_InitPhy);	*/
	p_phyops->IsPhyConnected = dm9161_IsPhyConnected;
/*printf("dm9161_IsPhyConnected: %x\n",dm9161_IsPhyConnected);		*/
	p_phyops->GetLinkSpeed = dm9161_GetLinkSpeed;
/*printf("dm9161_GetLinkSpeed: %x\n",dm9161_GetLinkSpeed);	*/
	p_phyops->AutoNegotiate = dm9161_AutoNegotiate;
/*printf("dm9161_AutoNegotiate: %x\n",dm9161_AutoNegotiate);		*/
}

#endif	/* CONFIG_COMMANDS & CFG_CMD_NET */

#endif	/* CONFIG_DRIVER_ETHER */
